/*$Id: nscgi-auth.c,v 1.1 1996/10/07 20:20:09 brian Exp $

  Netscape Fastrack CGI Authorization Patch

  Fixes a problem in Netscape FastTrack servers that prevents
  CGI scripts from recieving the HTTP_AUTHORIZATION header in
  their environments.

     Copyright 

       Copyright 1996 Digital Creations, L.C., 910 Princess Anne
       Street, Suite 300, Fredericksburg, Virginia 22401 U.S.A. All
       rights reserved.  Copyright in this software is owned by DCLC,
       unless otherwise indicated. Permission to use, copy and
       distribute this software is hereby granted, provided that the
       above copyright notice appear in all copies and that both that
       copyright notice and this permission notice appear. Note that
       any product, process or technology described in this software
       may be the subject of other Intellectual Property rights
       reserved by Digital Creations, L.C. and are not licensed
       hereunder.

     Trademarks 

       Digital Creations & DCLC, are trademarks of Digital Creations, L.C..
       All other trademarks are owned by their respective companies. 

     No Warranty 

       The software is provided "as is" without warranty of any kind,
       either express or implied, including, but not limited to, the
       implied warranties of merchantability, fitness for a particular
       purpose, or non-infringement. This software could include
       technical inaccuracies or typographical errors. Changes are
       periodically made to the software; these changes will be
       incorporated in new editions of the software. DCLC may make
       improvements and/or changes in this software at any time
       without notice.

     Limitation Of Liability 

       In no event will DCLC be liable for direct, indirect, special,
       incidental, economic, cover, or consequential damages arising
       out of the use of or inability to use this software even if
       advised of the possibility of such damages. Some states do not
       allow the exclusion or limitation of implied warranties or
       limitation of liability for incidental or consequential
       damages, so the above limitation or exclusion may not apply to
       you.

    If you have questions regarding this software,
    contact:
   
      Brian Lloyd, brian@digicool.com
      Digital Creations L.C. (540) 371-6909
*/


/* NSAPI header files */
#include "base/util.h"
#include "base/shexp.h"
#include "base/pblock.h"
#include "base/session.h"
#include "frame/req.h"
#include "frame/http.h"
#include "base/ereport.h"
#include "frame/log.h"

/* Non-NSAPI */
#include <stdio.h>
#include <string.h>




/* Abstract-ify the mechanics of exporting dll functions */
#ifdef XP_WIN32
#define NSAPI_PUBLIC __declspec(dllexport)
#else /* !XP_WIN32 */
#define NSAPI_PUBLIC
#endif /* !XP_WIN32 */

#ifdef PERM_MALLOC
#define NS_PERM_MALLOC(x)     PERM_MALLOC(x)
#else
#define NS_PERM_MALLOC(x)     MALLOC(x)
#endif

#define NS_MALLOC(x)          MALLOC(x)
#define NS_STRDUP(x)          STRDUP(x)
#define NS_REALLOC(x, y)      REALLOC(x, y)
#define NS_FREE(x)            FREE(x)
#define NS_PERM_FREE(x)       PERM_FREE(x)
#define NS_PERM_STRDUP(x)     PERM_STRDUP(x)
#define NS_PERM_REALLOC(x, y) PERM_REALLOC(x, y)

#define NS_DEBUG(x)           if(_debug) { fprintf x; }
#define NS_VERSION            MAGNUS_VERSION_STRING
#define VERSION               "1.0"



static char copyright[]       = "(c)1996 Digital Creations, L.C.\n";
int _debug                    = 0;
int _config                   = 0;
char startup_msg[300];
int  startup_msg_length;



typedef struct reg_data_s
{  char *match;
} *reg_data;


typedef struct reg_entry_s
{  reg_data _data;
   struct reg_entry_s *next;
} *reg_entry;


/* Create global data structures */
reg_entry registry[256];








/* Called on NS server initialization -- reads in shell match expressions
   which, if matching a URL, will cause the HTTP_AUTHORIZATION header to
   be passed to a CGI script as HTTP_CGI_AUTHORIZATION */

NSAPI_PUBLIC int cgiauth_initialize(pblock *pb, Session *sn, Request *rq)
{ char *debug;
  char *use_config;
  reg_entry    *pregistry;
  reg_entry    new_entry;
  reg_data     new_data;

  startup_msg_length = util_sprintf(startup_msg,
                                    "NS CGI Authentication patch, v.%s\n%s",
                                     VERSION, copyright);
  printf (startup_msg);

  /* Check to see if debugging is on */
  debug = pblock_findval("debug", pb);
  if (debug && atoi(debug))
    { _debug = 1;
      NS_DEBUG((stderr, "cgiauth_initialize: debugging to stderr\n"));
      NS_FREE(debug);
    }
  ereport(LOG_INFORM,
          "cgiauth_initialize: Initializing CGI Authentication Patch");

  /* Check to see if we should allow all CGI scripts access to auth headers */
  use_config = pblock_findval("config", pb);
  if (use_config)
  {  _config = 1;
     ereport(LOG_INFORM, "cgiauth_initialize: Using configuration");
     /* Parse config here one day... */
     NS_FREE(use_config);
     return REQ_PROCEED;
  }

  /* Not using a config, so initialize '*' as a global match */
  pregistry = &registry[(unsigned short)'n'];
  while (*pregistry)
    {
      pregistry = &((*pregistry)->next);
    }

  new_data         = NS_PERM_MALLOC(sizeof(*new_data));
  new_data->match  = "*";
  new_entry = NS_PERM_MALLOC(sizeof(*new_entry));
  *pregistry = new_entry;
  new_entry->next  = NULL;
  new_entry->_data = new_data;

  return REQ_PROCEED;
}



NSAPI_PUBLIC int cgiauth_nametrans(pblock *pb, Session *sn, Request *rq)
{
   char *aheader = pblock_findval("authorization", rq->headers);

   if (aheader)
   { /* There was a header, so see if it matches any of our allow entries.*/
     char *ppath = pblock_findval("ppath", rq->vars);
     reg_entry allentries = registry[(unsigned short)'n'];
     while (allentries)
     {  if (shexp_cmp(ppath, allentries->_data->match) != 0)
          allentries = allentries->next;
        else
        {  pblock_nvinsert("CGI_AUTHORIZATION",
                            NS_STRDUP(aheader), rq->headers);
           return REQ_NOACTION;
        }
     }
   }
   /* Didn't find an auth header, so return. */
   return REQ_NOACTION;
}








